/* ssrga.h -- Header for Self-Similar Rayleigh Gans Approximation calculations
 
Author: Robin Hogan <r.j.hogan@ecmwf.int>
Copyright (C) 2016  European Centre for Medium Range Weather Forecasts 
Language: C99
 
The Self-Similar Rayleigh Gans Approximation (SSRGA) computes
scattering properties for ice aggregates; See Hogan et al. (QJRMS
2017) for full details of the method. Normal usage would be to first
call one of the dielectric_factor_* functions to compute the real and
imaginary parts of the effective dielectric factor "K" of the monomers
from which the aggregate is composed. The aspect ratio of the monomers
is provided at this point. Then pass "K" into one of the ssrga_*
functions to compute the backscatter cross-section, scattering
properties or the full phase function.

*/

#ifndef SSRGA_H
#define SSRGA_H 1

/* Define "real" and "real_complex" to be single or double
   precision */
#include "real.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
The behaviour of SSRGA is governed by seven parameters.  Some values
for these are estimated from simulated ice aggregates in Hogan and
Westbrook (JAS 2014) and Hogan et al. (QJRMS 2017). Typical values for
ice particles and provided in the SSRGA_ICE_DEFAULT_* preprocessor
variables, as now discussed.  These are not used in ssrga.c, but are
available to users in case they are useful.

The first parameter is the aspect ratio of the monomer crystals,
provided to the dielectric_factor_hexagonal function. This is
important because non-spherical monomers scatter and absorb
systematically more than spheres, even for particles smaller than the
wavelength. Hogan et al. (2017) considered aggregates of bullet
rosettes, for which a value of 4.0 (columnar crystals with length four
times their width) led to good agreement with DDA calculations.
 */
#define SSRGA_ICE_DEFAULT_MONOMER_ASPECT_RATIO 4.0

/*
The effective aspect ratio is the ratio between the dimension of the
particle in the direction of propagation, and the maximum dimension of
the particle. It is used in computing "d_eff" (input to the ssrga_*
functions below) from the maximum dimension which users may have their
data in terms of. For computing backscatter by a nadir- or
zenith-pointing radar, a value of around 0.6 has been found to fit the
data well (Hogan et al., JAMC 2012). For computing other scattering
parameters where the direction of propagation is from other
directions, it is better to consider randomly oriention, for which
Hogan et al. (2017) reported an effective aspect ratio (which they
denoted alpha_eff) of 0.83 for aggregates of bullet rosettes simulated
by the Westbrook aggregation model.
 */
#define SSRGA_ICE_DEFAULT_ASPECT_RATIO 0.60
#define SSRGA_ICE_DEFAULT_ALPHA_EFF    0.83

/*
For the other parameters directly provided to the ssrga_* functions
below, Hogan et al. (2017) reported the following values in the case
of the Westbrook aggregation model: kappa=0.09, beta=0.55, gamma=2.0,
zeta1=0.28. Note that the beta variable was actually found to vary
somewhat with size, but since this dependence was opposite for the
Nowell aggregate model, a single value is probably sufficient.
 */
#define SSRGA_ICE_DEFAULT_KAPPA 0.09
#define SSRGA_ICE_DEFAULT_BETA  0.55
#define SSRGA_ICE_DEFAULT_GAMMA 2.00
#define SSRGA_ICE_DEFAULT_ZETA1 0.28


/* Compute the real and imagnary parts of the dielectric factor for a
   spherical monomer using the Clausius-Mossotti formula. */
void
dielectric_factor_sphere(real refind_r, /* Real part refractive index */
			 real refind_i, /* Imag part refractive index */
			 real* k_r,     /* Real part dielectric factor */
			 real* k_i);    /* Imag part dielectric factor */

/* Compute the real and imaginary parts of the effective dielectric
   factor for a randomly oriented spheroidal monomers using Gans
   theory, where aspect<1 indicates oblate spheroids and aspect>1
   indicates prolate spheroids. */
void
dielectric_factor_spheroid(real refind_r, /* Real part refractive index */
			   real refind_i, /* Imag part refractive index */
			   real aspect,   /* Aspect ratio */
			   real* k_r,     /* Real part dielectric factor */
			   real* k_i);    /* Imag part dielectric factor */
  
/* Compute the real and imaginary parts of the effective dielectric
   factor for a randomly oriented hexagonal monomers using Westbrook
   (2012) theory, where aspect<1 indicates a hexagonal plate and
   aspect>1 indicates a hexagonal column. */
void
dielectric_factor_hexagonal(real refind_r,/* Real part refractive index */
			    real refind_i,/* Imag part refractive index */
			    real aspect,  /* Aspect ratio */
			    real* k_r,    /* Real part dielectric factor */
			    real* k_i);   /* Imag part dielectric factor */

/* Return the radar backscatter cross-section for the parameters
   provided, with units of "m2". To obtain backscatter per unit solid
   angle (in m2 sr-1), divide the result by 4pi. The meanings of
   kappa, beta, gamma and zeta1 are given in Hogan et al. (2017). The
   variable "d_eff" is equal to the maximum dimension multiplied by
   the effective aspect ratio (denoted alpha_eff by Hogan et
   al. 2017). */
real
ssrga_backscatter(real wavelength, /* Wavelength (m) */
		  real d_eff,      /* Particle dimension in propagation direction (m) */
		  real volume,     /* Volume of material (m3) */
		  real k_r,        /* Real part of dielectric param */
		  real k_i,        /* Imag part of dielectric param */
		  real kappa,      /* Kurtosis parameter */
		  real beta,       /* Power law prefactor */
		  real gamma,      /* Power law exponent (+ve) */
		  real zeta1);     /* Wavenumber-1 power scaling */

/* Return the radar backscatter cross-section, the scattering and
   absorption cross-sections, and the asymmetry factor, for the
   parameters provided. Except for backscatter, the scattering
   properties are computed assuming random particle orientation with
   respect to the direction of propagation of the incident
   radiation. If you require the backscatter for nadir or zenith
   incidence then you may wish to call ssrga_backscatter separately
   for that one variable. */
void
ssrga_cross_sections(real wavelength, /* Wavelength (m) */
		     real d_eff,      /* Particle dimension in propagation direction (m) */
		     real volume,     /* Volume of material (m3) */
		     real k_r,        /* Real part of dielectric_param */
		     real k_i,        /* Imag part of dielectric_param */
		     real kappa,      /* Kurtosis parameter */
		     real beta,       /* Power law prefactor */
		     real gamma,      /* Power law exponent (+ve) */
		     real zeta1,      /* Wavenumber-1 power scaling */
		     real* bscat_xs,  /* Backscatter cross section (m2) */
		     real* scat_xs,   /* Scattering cross section (m2) */
		     real* abs_xs,    /* Absorption cross section (m2) */
		     real* asym);     /* Asymmetry factor */

/* Return the radar backscatter cross-section, the scattering and
   absorption cross-sections, the asymmetry factor and the phase
   function, for the parameters provided. The phase function is
   normalized such that the integral from 0 to pi of the phase
   function multiplied by the sine of the scattering angle is
   one. Except for backscatter, the scattering properties are computed
   assuming random particle orientation with respect to the direction
   of propagation of the incident radiation. If you require the
   backscatter for nadir or zenith incidence then you may wish to call
   ssrga_backscatter separately for that one variable. */
void
ssrga_phase_function(real wavelength, /* Wavelength (m) */
		     real d_eff,      /* Particle dimension in propagation direction (m) */
		     real volume,     /* Volume of material (m3) */
		     real k_r,        /* Real part of dielectric_param */
		     real k_i,        /* Imag part of dielectric_param */
		     real kappa,      /* Kurtosis parameter */
		     real beta,       /* Power law prefactor */
		     real gamma,      /* Power law exponent (+ve) */
		     real zeta1,      /* Wavenumber-1 power scaling */
		     real* bscat_xs,  /* Backscatter cross section (m2) */
		     real* scat_xs,   /* Scattering cross section (m2) */
		     real* abs_xs,    /* Absorption cross section (m2) */
		     real* asym,      /* Asymmetry factor */
		     int nang,        /* Number of evenly-spaced scattering angles */
		     real pp[nang]);  /* Scattering phase function */

#ifdef __cplusplus
}
#endif

#endif
